@@ -2,7 +2,7 @@
 #
 # The MIT License (MIT)
 #
-# Copyright (c) Tavendo GmbH
+# Copyright (c) Crossbar.io Technologies GmbH
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -203,14 +203,24 @@ class BaseSession(ObservableMixin):
                     encrypted_payload = EncryptedPayload(msg.enc_algo, msg.enc_key, msg.enc_serializer, msg.payload)
                     decrypted_error, msg.args, msg.kwargs = self._keyring.decrypt(True, msg.error, encrypted_payload)
                 except Exception as e:
-                    log_msg = u"failed to decrypt application payload 1: {}".format(e)
-                    self.log.warn(log_msg)
-                    enc_err = ApplicationError(ApplicationError.ENC_DECRYPT_ERROR, log_msg, enc_algo=msg.enc_algo)
+                    self.log.warn("failed to decrypt application payload 1: {err}", err=e)
+                    enc_err = ApplicationError(
+                        ApplicationError.ENC_DECRYPT_ERROR,
+                        u"failed to decrypt application payload 1: {}".format(e),
+                        enc_algo=msg.enc_algo,
+                    )
                 else:
                     if msg.error != decrypted_error:
-                        log_msg = u"URI within encrypted payload ('{}') does not match the envelope ('{}')".format(decrypted_error, msg.error)
-                        self.log.warn(log_msg)
-                        enc_err = ApplicationError(ApplicationError.ENC_TRUSTED_URI_MISMATCH, log_msg, enc_algo=msg.enc_algo)
+                        self.log.warn(
+                            u"URI within encrypted payload ('{decrypted_error}') does not match the envelope ('{error}')",
+                            decrypted_error=decrypted_error,
+                            error=msg.error,
+                        )
+                        enc_err = ApplicationError(
+                            ApplicationError.ENC_TRUSTED_URI_MISMATCH,
+                            u"URI within encrypted payload ('{}') does not match the envelope ('{}')".format(decrypted_error, msg.error),
+                            enc_algo=msg.enc_algo,
+                        )
 
         if enc_err:
             return enc_err
@@ -324,7 +334,7 @@ class ApplicationSession(BaseSession):
         """
         self.join(self.config.realm)
 
-    def join(self, realm, authmethods=None, authid=None, authrole=None, authextra=None):
+    def join(self, realm, authmethods=None, authid=None, authrole=None, authextra=None, resumable=None, resume_session=None, resume_token=None):
         """
         Implements :func:`autobahn.wamp.interfaces.ISession.join`
         """
@@ -347,7 +357,15 @@ class ApplicationSession(BaseSession):
         self._goodbye_sent = False
 
         # send HELLO message to router
-        msg = message.Hello(realm, role.DEFAULT_CLIENT_ROLES, authmethods, authid, authrole, authextra)
+        msg = message.Hello(realm=realm,
+                            roles=role.DEFAULT_CLIENT_ROLES,
+                            authmethods=authmethods,
+                            authid=authid,
+                            authrole=authrole,
+                            authextra=authextra,
+                            resumable=resumable,
+                            resume_session=resume_session,
+                            resume_token=resume_token)
         self._transport.send(msg)
 
     def disconnect(self):
@@ -367,7 +385,7 @@ class ApplicationSession(BaseSession):
         """
         Implements :func:`autobahn.wamp.interfaces.ISession.is_attached`
         """
-        return self._session_id is not None
+        return self._transport is not None and self._session_id is not None
 
     def onUserError(self, fail, msg):
         """
@@ -431,8 +449,18 @@ class ApplicationSession(BaseSession):
                     self._realm = msg.realm
 
                 self._session_id = msg.session
+                self._router_roles = msg.roles
 
-                details = SessionDetails(self._realm, self._session_id, msg.authid, msg.authrole, msg.authmethod, msg.authprovider, msg.authextra)
+                details = SessionDetails(realm=self._realm,
+                                         session=self._session_id,
+                                         authid=msg.authid,
+                                         authrole=msg.authrole,
+                                         authmethod=msg.authmethod,
+                                         authprovider=msg.authprovider,
+                                         authextra=msg.authextra,
+                                         resumed=msg.resumed,
+                                         resumable=msg.resumable,
+                                         resume_token=msg.resume_token)
                 # firing 'join' *before* running onJoin, so that the
                 # idiom where you "do stuff" in onJoin -- possibly
                 # including self.leave() -- works properly. Besides,
@@ -577,7 +605,18 @@ class ApplicationSession(BaseSession):
                         invoke_kwargs = msg.kwargs if msg.kwargs else dict()
 
                         if handler.details_arg:
-                            invoke_kwargs[handler.details_arg] = types.EventDetails(publication=msg.publication, publisher=msg.publisher, publisher_authid=msg.publisher_authid, publisher_authrole=msg.publisher_authrole, topic=topic, enc_algo=msg.enc_algo)
+                            invoke_kwargs[handler.details_arg] = types.EventDetails(publication=msg.publication, publisher=msg.publisher, publisher_authid=msg.publisher_authid, publisher_authrole=msg.publisher_authrole, topic=topic, retained=msg.retained, enc_algo=msg.enc_algo)
+
+                        # FIXME: https://github.com/crossbario/autobahn-python/issues/764
+                        def _success(_):
+                            # Acknowledged Events -- only if we got the details header and
+                            # the broker advertised it
+                            if msg.x_acknowledged_delivery and self._router_roles["broker"].x_acknowledged_event_delivery:
+                                if self._transport:
+                                    response = message.EventReceived(msg.publication)
+                                    self._transport.send(response)
+                                else:
+                                    self.log.warn("successfully processed event with acknowledged delivery, but could not send ACK, since the transport was lost in the meantime")
 
                         def _error(e):
                             errmsg = 'While firing {0} subscribed under {1}.'.format(
@@ -585,7 +624,7 @@ class ApplicationSession(BaseSession):
                             return self._swallow_error(e, errmsg)
 
                         future = txaio.as_future(handler.fn, *invoke_args, **invoke_kwargs)
-                        txaio.add_callbacks(future, None, _error)
+                        txaio.add_callbacks(future, _success, _error)
 
                 else:
                     raise ProtocolError("EVENT received for non-subscribed subscription ID {0}".format(msg.subscription))
@@ -663,14 +702,25 @@ class ApplicationSession(BaseSession):
                                 encrypted_payload = EncryptedPayload(msg.enc_algo, msg.enc_key, msg.enc_serializer, msg.payload)
                                 decrypted_proc, msg.args, msg.kwargs = self._keyring.decrypt(True, proc, encrypted_payload)
                             except Exception as e:
-                                log_msg = u"failed to decrypt application payload 1: {}".format(e)
-                                self.log.warn(log_msg)
-                                enc_err = ApplicationError(ApplicationError.ENC_DECRYPT_ERROR, log_msg)
+                                self.log.warn(
+                                    "failed to decrypt application payload 1: {err}",
+                                    err=e,
+                                )
+                                enc_err = ApplicationError(
+                                    ApplicationError.ENC_DECRYPT_ERROR,
+                                    u"failed to decrypt application payload 1: {}".format(e),
+                                )
                             else:
                                 if proc != decrypted_proc:
-                                    log_msg = u"URI within encrypted payload ('{}') does not match the envelope ('{}')".format(decrypted_proc, proc)
-                                    self.log.warn(log_msg)
-                                    enc_err = ApplicationError(ApplicationError.ENC_TRUSTED_URI_MISMATCH, log_msg)
+                                    self.log.warn(
+                                        "URI within encrypted payload ('{decrypted_proc}') does not match the envelope ('{proc}')",
+                                        decrypted_proc=decrypted_proc,
+                                        proc=proc,
+                                    )
+                                    enc_err = ApplicationError(
+                                        ApplicationError.ENC_TRUSTED_URI_MISMATCH,
+                                        u"URI within encrypted payload ('{}') does not match the envelope ('{}')".format(decrypted_proc, proc),
+                                    )
 
                     if msg.progress:
                         # process progressive call result
@@ -749,14 +799,26 @@ class ApplicationSession(BaseSession):
                                     encrypted_payload = EncryptedPayload(msg.enc_algo, msg.enc_key, msg.enc_serializer, msg.payload)
                                     decrypted_proc, msg.args, msg.kwargs = self._keyring.decrypt(False, proc, encrypted_payload)
                                 except Exception as e:
-                                    log_msg = u"failed to decrypt INVOCATION payload: {}".format(e)
-                                    self.log.warn(log_msg)
-                                    enc_err = ApplicationError(ApplicationError.ENC_DECRYPT_ERROR, log_msg)
+                                    self.log.warn(
+                                        "failed to decrypt INVOCATION payload: {err}",
+                                        err=e,
+                                    )
+                                    enc_err = ApplicationError(
+                                        ApplicationError.ENC_DECRYPT_ERROR,
+                                        "failed to decrypt INVOCATION payload: {}".format(e),
+                                    )
                                 else:
                                     if proc != decrypted_proc:
-                                        log_msg = u"URI within encrypted INVOCATION payload ('{}') does not match the envelope ('{}')".format(decrypted_proc, proc)
-                                        self.log.warn(log_msg)
-                                        enc_err = ApplicationError(ApplicationError.ENC_TRUSTED_URI_MISMATCH, log_msg)
+                                        self.log.warn(
+                                            "URI within encrypted INVOCATION payload ('{decrypted_proc}') "
+                                            "does not match the envelope ('{proc}')",
+                                            decrypted_proc=decrypted_proc,
+                                            proc=proc,
+                                        )
+                                        enc_err = ApplicationError(
+                                            ApplicationError.ENC_TRUSTED_URI_MISMATCH,
+                                            u"URI within encrypted INVOCATION payload ('{}') does not match the envelope ('{}')".format(decrypted_proc, proc),
+                                        )
 
                         if enc_err:
                             # when there was a problem decrypting the INVOCATION payload, we obviously can't invoke
@@ -823,8 +885,10 @@ class ApplicationSession(BaseSession):
                                             else:
                                                 encrypted_payload = self._keyring.encrypt(False, proc, [res])
                                         except Exception as e:
-                                            log_msg = u"failed to encrypt application payload: {}".format(e)
-                                            self.log.warn(log_msg)
+                                            self.log.warn(
+                                                "failed to encrypt application payload: {err}",
+                                                err=e,
+                                            )
 
                                 if encrypted_payload:
                                     reply = message.Yield(msg.request,
